"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * @class
 * @classdesc a base class that is to be extended by all the view classes
 * */
var AdharaView = function () {

    /**
     * @constructor
     * @param {AdharaView} parentViewInstance - parent view instance that is to be passed to render a view inside another.
     * */
    function AdharaView(parentViewInstance) {
        _classCallCheck(this, AdharaView);

        this.parentView = parentViewInstance;
        Adhara.instances[this.constructor.name] = this;
        this._data = null;
    }

    _createClass(AdharaView, [{
        key: "fetchData",


        /**
         * @function
         * @instance
         * @description hook to make API calls, data change event will be triggered on successful API call.
         * By default no API call will be made and dataChange method will be called right away.
         * */
        value: function fetchData() {
            this.handleDataChange();
        }
    }, {
        key: "handleDataChange",
        value: function handleDataChange(new_data) {
            this.dataChange(new_data);
            this.render();
        }
    }, {
        key: "dataChange",
        value: function dataChange(new_data) {
            this._data = new_data;
        }
    }, {
        key: "_getHTML",
        value: function _getHTML(template) {
            return HandlebarUtils.execute(template || this.template, this);
        }
    }, {
        key: "_getParentContainer",
        value: function _getParentContainer() {
            var container = this.parentView.contentContainer;
            if (typeof container === "string") {
                return container;
            }
            return container[this.constructor.name];
        }
    }, {
        key: "render",
        value: function render() {
            var container = document.querySelector(this._getParentContainer());
            container.innerHTML = this._getHTML();
            setTimeout(this.format(container), 0);
            this.renderSubViews();
        }

        /**
         * @function
         * @getter
         * @returns {String} A CSS selector inside which child views are to be rendered.
         * */

    }, {
        key: "renderSubViews",
        value: function renderSubViews() {
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = this.subViews[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var sub_view = _step.value;

                    var view = Adhara.getView(sub_view, this);
                    view.render();
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }
        }
    }, {
        key: "format",
        value: function format(container) {
            //Control the DOM elements after rendering
        }
    }, {
        key: "data",
        get: function get() {
            return this._data || {};
        }
    }, {
        key: "template",
        get: function get() {
            if (AdharaRouter.getCurrentPageName()) {
                return AdharaRouter.getCurrentPageName().replace(/_/g, "-");
            } else {
                var className = this.constructor.name.toLowerCase();
                if (className.endsWith("view")) {
                    return className.slice(0, className.length - 4);
                } else {
                    return className;
                }
            }
        }

        //TODO remove it completely...

    }, {
        key: "contentSelector",
        get: function get() {
            throw new Error("Not implemented");
        }
    }, {
        key: "contentContainer",
        get: function get() {
            throw new Error("implement this method");
        }

        /**
         * @typedef {Object} SubView - Sub view configuration
         * @property {String} container_selector - container's CSS selector
         * @property {class} view - view class reference of the sub view
         * */

        /**
         * @function
         * @instance
         * @returns Array<SubView>
         * */

    }, {
        key: "subViews",
        get: function get() {
            return [];
        }
    }]);

    return AdharaView;
}();
//# sourceMappingURL=View.js.map